@@ -30,11 +30,10 @@ martCheck = function(mart, biomart = NULL){
         stop("You must provide a valid Mart object. To create a Mart object use the function: useMart.  Check ?useMart for more information.")
     }
     if(!is.null(biomart)){
-        #   martcheck = strsplit(martBM(mart),"_", fixed = TRUE, useBytes = TRUE)[[1]][1]
         martcheck = martBM(mart)
         bmok = FALSE
-        for(k in 1:length(biomart)){
-            if(martcheck[1] == biomart[k]){	
+        for(k in seq_along(biomart)) {
+            if(martcheck[1] == biomart[k]) {	
                 bmok = TRUE
             }
         }		    
@@ -57,9 +56,24 @@ checkWrapperArgs = function(id, type, mart){
 bmRequest <- function(request, verbose = FALSE){
     if(verbose) writeLines(paste("Attempting web service request:\n",request, sep=""))
 
-    result <- httr::GET(request, content_type("text/plain")
-                        #, set_cookies(.cookies = c(redirect_mirror = 'no'))
-                        )
+    result <- tryCatch(httr::GET(request, content_type("text/plain"), timeout(10)),
+                       error = function(c) { "timeout" } )
+    
+    tryAgain <- any(result == "timeout") || httr::status_code(result) == 500
+    
+    ## try an alternative mirror if ensembl returns 500
+    if(tryAgain && grepl("ensembl", request)) {
+        mirrors <- c("www", "uswest", "useast", "asia")
+        subdomain <- stringr::str_match(request, "://([a-z]{3,5})\\.")[1,2]
+        mirror_option <- sample(mirrors[!mirrors %in% subdomain], size = 1)
+        message("Ensembl site unresponsive, trying ", mirror_option, " mirror")
+        request2 <- str_replace(request, pattern = "://([a-z]{3,5})\\.", 
+                                replacement = paste0("://", mirror_option, "."))
+        result <- httr::GET(request2, content_type("text/plain"))
+    } else { ## this isn't ensembl run the query again so we can get the error 
+        result <- httr::GET(request, content_type("text/plain"))
+    }
+    
     stop_for_status(result)
     result2 <- content(result, encoding = "UTF-8")
     if(is.na(result2)) {
@@ -430,10 +444,6 @@ useDataset <- function(dataset, mart, verbose = FALSE){
     return( mart )
 }
 
-## getName
-
-getName <- function(x, pos) if(is.null(x[[pos]])) NA else x[[pos]] 
-
 ## listAttributes
 
 listAttributes <- function(mart, page, what = c("name","description","page")) {
@@ -529,13 +539,16 @@ getBM <- function(attributes, filters = "", values = "", mart, curl = NULL,
     
     ## determine if we should use the results cache
     if(useCache) {
-        cache <- rappdirs::user_cache_dir(appname="biomaRt")
+        cache <- Sys.getenv(x = "BIOMART_CACHE", 
+                            unset = rappdirs::user_cache_dir(appname="biomaRt"))
         bfc <- BiocFileCache::BiocFileCache(cache, ask = FALSE)
     }
     hash <- .createHash(mart, attributes, filters, values, uniqueRows, bmHeader)
     if( useCache && .checkCache(bfc, hash) ) {
         
-        message("Cache found")
+        if(verbose) {
+            message("Cache found")
+        }
         cache_hits <- bfcquery(bfc, hash, field = "rname")
         if(nrow(cache_hits) > 1) {
             stop("Multiple cache results found")
@@ -547,7 +560,7 @@ getBM <- function(attributes, filters = "", values = "", mart, curl = NULL,
 
     } else { 
     
-    ## force the query to return the 'english text' header names with the result
+    ## force the query to return the 'descriptive text' header names with the result
     ## we use these later to match and order attribute/column names    
     callHeader <- TRUE
     xmlQuery = paste0("<?xml version='1.0' encoding='UTF-8'?><!DOCTYPE Query><Query  virtualSchemaName = '",
@@ -586,6 +599,7 @@ getBM <- function(attributes, filters = "", values = "", mart, curl = NULL,
                                width = options()$width - 10,
                                format = "Batch submitting query [:bar] :percent eta: :eta")
         pb$tick(0)
+        on.exit( pb$terminate() )
     }
     
     ## we submit a query for each chunk of the filter list
@@ -622,10 +636,7 @@ getBM <- function(attributes, filters = "", values = "", mart, curl = NULL,
     }
     ## collate results
     result <- do.call('rbind', resultList)
-    if(length(filterXmlList) > 1) {
-        pb$terminate()
-    }
-    
+
     if(useCache) {
         tf <- tempfile()
         save(result, file = tf)
